package org.example.world;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.fail;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

public class CityTest
{
	private City testCityObject;

	@BeforeEach
	public void setUp() throws Exception
	{
		testCityObject = new City();
	}

	@Test
	public void testInitialId()
	{
		int expected = 1;
		int actual = testCityObject.getId();
		assertEquals(expected, actual);
	}
	
	@Test
	public void testValidId()
	{
		int[] good = {1,23,432,3,8,29292};
		for (int current : good)
		{
			testCityObject.setId(current);
		}
	}
	
	@Test
	public void testZeroId()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setId(0));
	}
	
	@Test
	public void testNegitiveId()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setId(-1));
	}

	@Test
	public void testInitialName()
	{
		String expected = "Kabul";
		String actual = testCityObject.getName();
		assertEquals(expected, actual);
	}
	
	@Test
	public void testNullName()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setName(null));
	}
	
	@Test
	public void testEmptyName()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setName("  "));
	}
	
	@Test
	public void testTooLongName()
	{
		String bad = RandomStringUtils.randomAlphanumeric(36);
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setName(bad));
	}
	
	@Test
	public void testName()
	{
		String good = "Boston";
		testCityObject.setName(good);
	}
	
	@Test
	public void testInitialCountryCode()
	{
		String expected = "AFG";
		String actual = testCityObject.getCountryCode();
		assertEquals(expected, actual);
	}
	
	@Test
	public void testNullCountryCode()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setCountryCode(null));
	}
	
	@Test
	public void testWrongSizeCountryCode()
	{
		String[] bad = {"", "X", "XX", "XXXX", "XXXXX"};
		for (int i = 0; i < bad.length; i++)
		{
			try
			{
				testCityObject.setCountryCode(bad[i]);
				fail("Setting CountryCode to " + bad[i] + " did not fail as planned");
			}
			catch (IllegalArgumentException oops)
			{
				String expected = "CountryCode must 3 chars in length";
				String actual = oops.getMessage();
				assertEquals(expected, actual);
			}
		}
	}
	
	@Test
	public void testCountryCode()
	{
		String good = "USA";
		testCityObject.setName(good);
	}
	
	@Test
	public void testInitialDistrict()
	{
		String expected = "Kabol";
		String actual = testCityObject.getDistrict();
		assertEquals(expected, actual);
	}
	
	@Test
	public void testNullDistrict()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setDistrict(null));
	}
	
	@Test
	public void testEmptyDistrict()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setDistrict("  "));
	}
	
	@Test
	public void testTooLongDistrict()
	{
		String bad = RandomStringUtils.randomAlphanumeric(21);
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setDistrict(bad));
	}
	
	@Test
	public void testDistrict()
	{
		String good = "West Coast";
		testCityObject.setDistrict(good);
	}
	
	@Test
	public void testInitialPopulation()
	{
		int exptected = 1780000;
		int actual = testCityObject.getPopulation();
		assertEquals(exptected, actual);
	}
	
	@Test
	public void testValidPopulation()
	{
		int[] good = {312921300, 213213, 10, 3218};
		for (int current : good)
		{
			testCityObject.setPopulation(current);
		}
	}
	
	@Test
	public void testZeroPopulation()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setPopulation(0));
	}
	
	@Test
	public void testNegitivePopulation()
	{
		assertThrows(IllegalArgumentException.class, () ->
			testCityObject.setPopulation(-1));
	}
	
	@Test
	public void testToString()
	{
		String expected = "City [id=1, name=Kabul, countryCode=AFG, district=Kabol, population=1780000]";
		String actual = testCityObject.toString();
		assertEquals(expected, actual);
	}
	
	@Test
	public void testEquals()
	{
		City testCity1 = new City();
		testCity1.setId(21);
		testCity1.setName("Denver");
		testCity1.setCountryCode("USA");
		testCity1.setPopulation(2345600);
		
		City testCity2 = new City();
		testCity2.setId(21);
		testCity2.setName("Denver");
		testCity2.setCountryCode("USA");
		testCity2.setPopulation(2345600);
			
		assertTrue(testCity1.equals(testCity2));
	}
	
	@Test
	public void testNotEquals()
	{
		City testCity1 = new City();
		testCity1.setId(22);
		testCity1.setName("Denver");
		testCity1.setCountryCode("USA");
		testCity1.setPopulation(2345600);
		
		City testCity2 = new City();
		testCity2.setId(21);
		testCity2.setName("Denver");
		testCity2.setCountryCode("USA");
		testCity2.setPopulation(2345600);
			
		assertFalse(testCity1.equals(testCity2));
	}
	
	@SuppressWarnings("unlikely-arg-type")
	@Test
	public void testDifferentObjectEquals()
	{
		City testCity1 = new City();
		testCity1.setId(22);
		testCity1.setName("Denver");
		testCity1.setCountryCode("USA");
		testCity1.setPopulation(2345600);
		
		String testString = new String("Test");
		
		assertFalse(testCity1.equals(testString));
	}
	
	@Test
	public void testHashCode()
	{
		City testCity1 = new City();
		testCity1.setId(21);
		testCity1.setName("Denver");
		testCity1.setCountryCode("USA");
		testCity1.setPopulation(2345600);
		
		City testCity2 = new City();
		testCity2.setId(21);
		testCity2.setName("Denver");
		testCity2.setCountryCode("USA");
		testCity2.setPopulation(2345600);
			
		assertTrue(testCity1.hashCode() == testCity2.hashCode());
	}
}
